package documents {
	import com.zehfernando.display.debug.StatGraph;
	import com.zehfernando.display.drawPlane;
	import com.zehfernando.display.shapes.Circle;
	import com.zehfernando.utils.DebugUtils;

	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.KeyboardEvent;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;

	[SWF(width="800", height="600", frameRate="60", backgroundColor="#ffffff")]

	/**
	 * @author zeh
	 */
	public class TriangleTest extends Sprite {

		// Assets
		[Embed(source="/../embed/floyd_mipmap.jpg")]
		public static const IMAGE_FLOYD:Class;
		
		// Properties
		protected var fillBitmap:BitmapData;
		protected var fillBitmap2:BitmapData;
		protected var triangleSprite:Sprite;
		
		protected var controlOn:Boolean;
		protected var gridOn:Boolean;
		
		protected var p1:Circle;
		protected var p2:Circle;
		protected var p3:Circle;
		protected var p4:Circle
;
		// ================================================================================================================
		// CONSTRUCTOR ----------------------------------------------------------------------------------------------------

		public function TriangleTest() {
			
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align = StageAlign.TOP_LEFT;
			addChild(new StatGraph());
			
			controlOn = false;
			gridOn = false;
			
			var tt:TextField = new TextField();
			tt.text = "Drag corners to move plane\nX: toggle control markers\nZ: toggle quality reference grid";
			tt.embedFonts = false;
			tt.autoSize = TextFieldAutoSize.LEFT;
			tt.selectable = false;
			tt.x = 12;
			tt.y = 72;
			addChild(tt);
			
			// http://www.senocular.com/flash/tutorials/flash10drawingapi/
			// http://help.adobe.com/en_US/ActionScript/3.0_ProgrammingAS3/WS509D19BB-239B-4489-B965-844DDA611AE7.html
			
			var floyd:Bitmap = new IMAGE_FLOYD();
			fillBitmap = floyd.bitmapData;
			
			triangleSprite = new Sprite();
			addChild(triangleSprite);
			
			p1 = new Circle(20);
			p1.alpha = 0.4;
			p1.x = 200;
			p1.y = 200;
			addChild(p1);
			DebugUtils.makeDraggable(p1, updateTriangles, updateTriangles);

			p2 = new Circle(20);
			p2.alpha = 0.4;
			p2.x = 500;
			p2.y = 200;
			addChild(p2);
			DebugUtils.makeDraggable(p2, updateTriangles, updateTriangles);

			p3 = new Circle(20);
			p3.alpha = 0.4;
			p3.x = 200;
			p3.y = 425;
			addChild(p3);
			DebugUtils.makeDraggable(p3, updateTriangles, updateTriangles);

			p4 = new Circle(20);
			p4.alpha = 0.4;
			p4.x = 500;
			p4.y = 425;
			addChild(p4);
			DebugUtils.makeDraggable(p4, updateTriangles, updateTriangles);

			updateBitmap();
			updateTriangles();
			
			stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
		}


		// ================================================================================================================
		// INTERNAL INTERFACE ---------------------------------------------------------------------------------------------

		protected function updateBitmap(): void {
			
			if (Boolean(fillBitmap2)) fillBitmap2.dispose();
			fillBitmap2 = fillBitmap.clone();
			
			if (gridOn) {
				var cols:uint = 4;
				var rows:uint = 4;
				var gridWeight:int = 2;
				var i:int;
	
				for (i = 0; i <= cols; i++) {
					fillBitmap2.fillRect(new Rectangle((i/cols) * fillBitmap2.width - gridWeight/2, 0, gridWeight, fillBitmap2.height), 0x000000);	
				}
				for (i = 0; i <= rows; i++) {
					fillBitmap2.fillRect(new Rectangle(0, (i/rows) * fillBitmap2.width - gridWeight/2, fillBitmap2.width, gridWeight), 0x000000);	
				}
			}
		}

		protected function updateTriangles(): void {
			var pt1:Point = new Point(p1.x, p1.y);
			var pt2:Point = new Point(p2.x, p2.y);
			var pt3:Point = new Point(p3.x, p3.y);
			var pt4:Point = new Point(p4.x, p4.y);
			drawPlane(graphics, fillBitmap2, pt1, pt2, pt3, pt4);
			
			if (controlOn) {
				graphics.endFill();

				// Reference diagonal
				graphics.lineStyle(1, 0x000000, 0.3);
				graphics.moveTo(pt1.x, pt1.y);
				graphics.lineTo(pt4.x, pt4.y);

				// First triangle
				graphics.lineStyle(2, 0x000000, 0.75);
				graphics.moveTo(pt1.x, pt1.y);
				graphics.lineTo(pt2.x, pt2.y);
				graphics.lineTo(pt3.x, pt3.y);
				graphics.lineTo(pt1.x, pt1.y);
				
				// Second triangle
				graphics.moveTo(pt3.x, pt3.y);
				graphics.lineTo(pt2.x, pt2.y);
				graphics.lineTo(pt4.x, pt4.y);
				graphics.lineTo(pt3.x, pt3.y);
				
			}
		}

		// ================================================================================================================
		// EVENT INTERFACE ------------------------------------------------------------------------------------------------
	
		protected function onKeyDown(e:KeyboardEvent): void {
			if (e.keyCode == 90) {
				// z
				gridOn = !gridOn;
				updateBitmap();
				updateTriangles();
			}
			if (e.keyCode == 88) {
				// x
				controlOn = !controlOn;
				updateTriangles();
			}
		}
	}
}
